import * as util from "util";
import * as fs from "fs";
import { EventEmitter } from "events";
import * as path from "path";
import { Lock, Observable } from "@hibas123/utils";
import { Adapter, LoggingTypes, Message } from "./types";
import { ConsoleWriter } from "./consolewriter";
import { LoggingBaseOptions, LoggingBase } from "./base";
import { LoggingFiles } from "./filewriter";

const OriginalErrorStackFunction = (<any>Error.prototype).prepareStackTrace

export const Colors = {
   Reset: "\x1b[0m",
   Bright: "\x1b[1m",
   Dim: "\x1b[2m",
   Underscore: "\x1b[4m",
   Blink: "\x1b[5m",
   Reverse: "\x1b[7m",
   Hidden: "\x1b[8m",

   FgBlack: "\x1b[30m",
   FgRed: "\x1b[31m",
   FgGreen: "\x1b[32m",
   FgYellow: "\x1b[33m",
   FgBlue: "\x1b[34m",
   FgMagenta: "\x1b[35m",
   FgCyan: "\x1b[36m",
   FgWhite: "\x1b[37m",

   BgBlack: "\x1b[40m",
   BgRed: "\x1b[41m",
   BgGreen: "\x1b[42m",
   BgYellow: "\x1b[43m",
   BgBlue: "\x1b[44m",
   BgMagenta: "\x1b[45m",
   BgCyan: "\x1b[46m",
   BgWhite: "\x1b[47m"
}

export interface LoggingOptions extends LoggingBaseOptions {
   files: boolean | {
      /**
       * Filename/path of the logfile. Skip if generated with name
       */
      logfile: string;
      /**
       * Filename/path of the logfile. Skip if generated with name
       */
      errorfile: string;
   }
}

export class LoggingExtended extends LoggingBase {
   constructor(config: Partial<LoggingOptions>) {
      super(config);

      if (typeof config.files === "boolean") {
         if (config.files === true) {
            let name = "";
            if (config.name)
               name = "." + config.name;
            config.files = {
               logfile: `./logs/all${name}.log`,
               errorfile: `./logs/error${name}.log`
            }
         } else {
            config.files = undefined;
         }
      }

      if (typeof config.files !== "boolean" && config.files !== undefined && (config.files.logfile || config.files.errorfile)) {

         if (config.files.logfile) {
            this.addAdapter(new LoggingFiles(config.files.logfile));
         }

         if (config.files.errorfile) {
            this.addAdapter(new LoggingFiles(config.files.errorfile, true));
         }
      }
   }
}

export let Logging: LoggingExtended = undefined;
if (process.env.LOGGING_NO_DEFAULT !== "true") {
   Logging = new LoggingExtended();
}
export default Logging;



