import { LoggingBaseOptions, LoggingBase } from "./base";
import { LoggingFiles } from "./filewriter";

export const Colors = {
   Reset: "\x1b[0m",
   Bright: "\x1b[1m",
   Dim: "\x1b[2m",
   Underscore: "\x1b[4m",
   Blink: "\x1b[5m",
   Reverse: "\x1b[7m",
   Hidden: "\x1b[8m",

   FgBlack: "\x1b[30m",
   FgRed: "\x1b[31m",
   FgGreen: "\x1b[32m",
   FgYellow: "\x1b[33m",
   FgBlue: "\x1b[34m",
   FgMagenta: "\x1b[35m",
   FgCyan: "\x1b[36m",
   FgWhite: "\x1b[37m",

   BgBlack: "\x1b[40m",
   BgRed: "\x1b[41m",
   BgGreen: "\x1b[42m",
   BgYellow: "\x1b[43m",
   BgBlue: "\x1b[44m",
   BgMagenta: "\x1b[45m",
   BgCyan: "\x1b[46m",
   BgWhite: "\x1b[47m"
}

export interface LoggingOptions extends LoggingBaseOptions {
   files: boolean | {
      /**
       * Filename/path of the logfile. Skip if generated with name
       */
      logfile: string;
      /**
       * Filename/path of the logfile. Skip if generated with name
       */
      errorfile: string;
   }
}

export class LoggingExtended extends LoggingBase {
   constructor(config: Partial<LoggingOptions> = {}) {
      super(config);

      if (typeof config === "string" || config.files !== false) {
         let logfile: string;
         let errorfile: string;
         if (typeof config.files === "object") {
            logfile = config.files.logfile;
            errorfile = config.files.errorfile;
         } else {
            let name = "." + this.name;
            logfile = `./logs/all${name}.log`;
            errorfile = `./logs/error${name}.log`;
         }

         if (logfile) {
            this.addAdapter(new LoggingFiles(logfile));
         }

         if (errorfile) {
            this.addAdapter(new LoggingFiles(errorfile, true));
         }
      }
   }
}

export let Logging: LoggingExtended = undefined;
if (process.env.LOGGING_NO_DEFAULT !== "true") {
   Logging = new LoggingExtended();
}
export default Logging;



